------------------------------------------------------------------------------
--                                 Ada2Java                                 --
--                                                                          --
--                     Copyright (C) 2004-2014, AdaCore                     --
--                                                                          --
-- This is free software;  you can redistribute it  and/or modify it  under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 3,  or (at your option) any later ver- --
-- sion.  This software is distributed in the hope  that it will be useful, --
-- but WITHOUT ANY WARRANTY;  without even the implied warranty of MERCHAN- --
-- TABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public --
-- License for  more details.  You should have  received  a copy of the GNU --
-- General  Public  License  distributed  with  this  software;   see  file --
-- COPYING3.  If not, go to http://www.gnu.org/licenses for a complete copy --
-- of the license.                                                          --
------------------------------------------------------------------------------

--  This is a placeholder for the routine that checks tool parameters. Because
--  all the checks are tool-specific, this placeholder only set up the Arg_List
--  global variable.

with Ada.Strings.Wide_Unbounded; use Ada.Strings.Wide_Unbounded;

with Ada2Java;        use Ada2Java;
with Ada2Java.Config; use Ada2Java.Config;
with Ada2Java.Kernel; use Ada2Java.Kernel;

with Ada2Java.Projects;

separate (ASIS_UL.Environment)
procedure Check_Parameters is
begin
   Trace ("Check Parameters", Ada2Java.Debug);
   Project_Support_Type := Use_Tmp_Project_File;

   if Project_File /= null then
      Process_Project_File (Project_File.all);
   end if;

-- 8<--8<--8<--8<--8<--8<--8<--8<--8<--8<--8<--8<--
   --  This is needed for multiple files driver only

   Read_Args_From_Temp_Storage
     (Duplication_Report =>
      not Ada2Java.Projects.Is_Specified (Ada2Java.Projects.Ada2Java_Prj),
      Arg_Project => Ada2Java.Projects.Ada2Java_Prj);

   if Last_Source < First_SF_Id then
      Error ("No input source file set");
      --  A call to Brief_Help shoul be here, but this is tool-specific
      raise Parameter_Error;
   end if;

   Total_Sources := Natural (Last_Source);
   Sources_Left  := Total_Sources;

   Set_Arg_List;
-- >8-->8-->8-->8-->8-->8-->8-->8-->8-->8-->8-->8--

   declare
      Default_Config : Configuration;
   begin
      Set_Java_Base_Package
        (Default_Config, To_Wide_String (Default_Base_Package));
      Set_Java_Class_Path
        (Default_Config, To_Wide_String (Default_Class_Path));
      Set_Ada_Output_Directory
        (Default_Config, To_Wide_String (Default_Ada_Base_Folder));
      Set_Default_Configuration (Ada2Java.The_Kernel, Default_Config);
   end;

end Check_Parameters;
